@@ -646,6 +646,12 @@ register unsigned char *kptr;       /* unsigned char[24] */
 #endif  /* D3_DES */
 #endif  /* D2_DES */
 
+void close_des(void) {
+	zap(&KnL[0], sizeof(unsigned long)*32);
+	zap(&KnR[0], sizeof(unsigned long)*32);
+	zap(&Kn3[0], sizeof(unsigned long)*32);
+}
+
 /* Validation sets:
  *
  * Single-length key, single-length plaintext -
