@@ -9,6 +9,8 @@
 main(int argc,char *argv[])
 {
     char pubfile[128]; unsigned int i,k;
+	char hash[16];
+	unsigned char idstring[128];
     unsigned char key[48], cipher[MAXC];
     FILE *public;
     MP_INT *e,*n,*c;
@@ -27,6 +29,28 @@ main(int argc,char *argv[])
         exit(1);
     }
 
+    if (!read_number(public,n)) {
+		fprintf(stderr, "problem reading %s - 1\n", pubfile);
+		exit(1);
+	}
+    if (!read_number(public,e)) {
+		fprintf(stderr, "problem reading %s - 2\n", pubfile);
+		exit(1);
+	}
+	
+	if (!read_string(public,idstring)) {
+		fprintf(stderr, "warning: public key in %s is not self signed\n",pubfile);
+	}
+	else {
+		mdpkid(n,e,idstring,hash);
+		if (!read_number(public,c) ||  !vsign(c,hash,n,e)) {
+			fprintf(stderr, "public key in %s is self signed with an INVALID signature\n",pubfile);
+			fprintf(stderr, "public key corruption detected\n");
+			exit(1);
+		}
+		fprintf(stderr, "key is self signed -- encrypting for: %s\n", idstring);
+	}
+
     initrandom();
 	for (i=2; i<argc;i++)
 		if (argv[i] && (k=strlen(argv[i])))  {
@@ -38,11 +62,9 @@ main(int argc,char *argv[])
     for (i=0;i<48;i++)
         key[i] = randombyte();
 
-    read_number(public,n);
-    read_number(public,e);
+
     encrypt_session_key(c,key,n,e);
     write_number(stdout,c);
     desfile(stdin,stdout,&key[0], &key[40], EN0);
 	zap(key,48); closerandom();
 }
-
