@@ -7,6 +7,42 @@
 #include "misc.h"
 #include "md5.h"
 
+int 
+write_string(FILE *f, unsigned char *string) 
+{
+	unsigned int length;
+	unsigned int i;
+	unsigned char clength;
+
+	length = strlen(string);
+	if (length > 127)
+		return 0;
+	clength = (unsigned char)length;
+
+	fputc(length,f);
+	for (i=0;i<length;i++)
+		fputc(*(string+i),f);
+	return 1;
+}
+
+int 
+read_string(FILE *f, unsigned char *buf) 
+{
+	int i; int length; int j;
+    length = fgetc(f);
+    if (length < 0 || length > 127)
+        return 0;
+	for (i=0;i<length;i++) {
+		j = fgetc(f);
+		if (j >= 0)
+			*(buf+i) = j;
+		else
+			return 0;
+	}
+	*(buf+i) = '\0';
+	return 1;
+}
+	
 void write_number(FILE *f ,MP_INT *x)
 {
     unsigned char buf[MAXC];
@@ -139,6 +175,22 @@ void mdstring(char *s, unsigned int len, unsigned char *digest)
   memcpy(digest, &mdContext.digest[0],16);
 }
 
+void 
+mdpkid(MP_INT *n, MP_INT *e, unsigned char *id, unsigned char *digest)
+{
+	unsigned char buf[MAXC];
+	MD5_CTX mdContext;
+	MD5Init (&mdContext);
+
+	mptcs(buf,MAXC,n);
+	MD5Update (&mdContext, buf, MAXC);
+	mptcs(buf,MAXC,e);
+	MD5Update (&mdContext, buf, MAXC);
+	MD5Update (&mdContext, id, strlen(id));
+	MD5Final (&mdContext);
+	memcpy(digest, &mdContext.digest[0],16);
+}
+
 /* 16 byte key - for private keys */
 static void
 desbuf(int mode, unsigned char *buf, unsigned int len,
