@@ -12,6 +12,10 @@
 #include <unistd.h>
 #include <termios.h>
 #include <stdio.h>
+#include <signal.h>
+
+static struct termios tty_orig,tty_new;
+static FILE *tty;
 
 #define RBUFLEN 65536
 
@@ -55,6 +59,72 @@ unsigned char b2[16];
 unsigned char b3[16];
 unsigned char b4[16];
 
+static void eh(int n)
+{
+    tcdrain(fileno(tty));
+    tcsetattr(fileno(tty), TCSAFLUSH, &tty_orig);
+	fprintf(stderr, "\nreceived signal %d -- execution aborted\n",n);
+	exit(1);
+}
+
+static void install_handlers()
+{
+#ifdef SIGINT
+signal(SIGINT, eh);
+#endif
+#ifdef SIGHUP
+signal(SIGHUP, eh);
+#endif
+signal(SIGHUP, eh);
+#ifdef SIGSEGV
+signal(SIGSEGV, eh);
+#endif
+#ifdef SIGBUS
+signal(SIGBUS,eh);
+#endif
+#ifdef SIGEMT
+signal(SIGEMT,eh);
+#endif
+#ifdef SIGILL
+signal(SIGILL, eh);
+#endif
+#ifdef SIGQUIT
+signal(SIGQUIT, eh);
+#endif
+#ifdef SIGTERM
+signal(SIGTERM, eh);
+#endif
+}
+static void restore_default_handlers()
+{
+#ifdef SIGINT
+signal(SIGINT, SIG_DFL);
+#endif
+#ifdef SIGHUP
+signal(SIGHUP, SIG_DFL);
+#endif
+signal(SIGHUP, SIG_DFL);
+#ifdef SIGSEGV
+signal(SIGSEGV, SIG_DFL);
+#endif
+#ifdef SIGBUS
+signal(SIGBUS,SIG_DFL);
+#endif
+#ifdef SIGEMT
+signal(SIGEMT,SIG_DFL);
+#endif
+#ifdef SIGILL
+signal(SIGILL, SIG_DFL);
+#endif
+#ifdef SIGQUIT
+signal(SIGQUIT, SIG_DFL);
+#endif
+#ifdef SIGTERM
+signal(SIGTERM, SIG_DFL);
+#endif
+}
+
+
 void initrandom()
 {
     struct timeval foo;
@@ -112,11 +182,10 @@ void seedrandom(unsigned char *buf, unsigned int len)
 unsigned char *getstring(int pwd)
 {
     struct timeval tv; struct timezone tz;
-    FILE *tty; int i;
+    int i;
     unsigned char *s;
     unsigned int len = 0;
 	long vdis;
-    struct termios tty_orig,tty_new;
 #ifdef GETRUSAGE
     struct rusage foo3;
 #endif
@@ -130,22 +199,21 @@ unsigned char *getstring(int pwd)
     }
 
     tcgetattr(fileno(tty), &tty_orig);
-
+	install_handlers();
+    memcpy(&(tty_new),&(tty_orig),sizeof(tty_orig));
 	if ((vdis = fpathconf(fileno(tty),_PC_VDISABLE)) < 0) {
 		fprintf(stderr, 
 			"warning: fpathconf error or _POSIX_VDISABLE not in effect\n");
 	}
-
-    memcpy(&(tty_new),&(tty_orig),sizeof(tty_orig));
-
-    tty_new.c_lflag &= ~(ECHO | ICANON);
-    tty_new.c_cc[VMIN] = 1;
-    tty_new.c_cc[VTIME] = 0;
-	if (vdis > 0) {
+	else  {
 		tty_new.c_cc[VINTR] = vdis;
 		tty_new.c_cc[VQUIT] = vdis;
 		tty_new.c_cc[VSUSP] = vdis;
 	}
+
+    tty_new.c_lflag &= ~(ECHO | ICANON);
+    tty_new.c_cc[VMIN] = 1;
+    tty_new.c_cc[VTIME] = 0;
     tcdrain(fileno(tty));
     tcsetattr(fileno(tty), TCSAFLUSH, &tty_new);
 
@@ -168,6 +236,9 @@ unsigned char *getstring(int pwd)
     while (! (i == '\n' || i == (-1)));
     tcdrain(fileno(tty));
     tcsetattr(fileno(tty), TCSAFLUSH, &tty_orig);
+	restore_default_handlers();
+	putc('\n', stderr);
+	fclose(tty);
     if (pwd) {
         s[len]=0;
         return(s);
