@@ -10,6 +10,7 @@ main(int argc,char *argv[])
 {
     MP_INT *e,*c,*n;
     char pubfile[128]; int i;
+	unsigned char idstring[128];
     unsigned char hash[16];
     unsigned char sig[MAXC];
     FILE *public; 
@@ -30,6 +31,18 @@ main(int argc,char *argv[])
     
     read_number(public,n);
     read_number(public,e);
+	if (!read_string(public,idstring)) {
+		fprintf(stderr, "warning: public key in %s is not self signed\n",pubfile);
+	}
+	else {
+		mdpkid(n,e,idstring,hash);
+		if (!read_number(public,c) ||  !vsign(c,hash,n,e)) {
+			fprintf(stderr, "public key in %s is self signed with an INVALID signature\n",pubfile);
+			fprintf(stderr, "public key corruption detected\n");
+			exit(1);
+		}
+		fprintf(stderr, "key is self signed -- verifying signature by %s\n", idstring);
+	}
     if (!(sigfile = fopen(argv[2],"rb")))  {
         fprintf(stderr, "could not open %s for read\n", argv[2]);
         exit(1);
@@ -39,10 +52,10 @@ main(int argc,char *argv[])
 
     mdfile(stdin, hash);    
     if (vsign(c,hash,n,e)) {
-        printf("signature verified\n");
+        printf("file signature verified\n");
     }
     else {
-        printf("signature did not verify\n");
+        printf("file signature did not verify\n");
         exit(1);
     }
     exit(0);
